/*
 * This file is part of SubLib, a library for managing movie subtitles.
 * Copyright (C) 2005-2006 Pedro Castro
 *
 * SubLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SubLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

using System;
using System.IO;
using System.Text;

namespace SubLib {
	
/// <summary>Represents the properties of subtitles.</summary>
/// <remarks>This class acts as a container which allows you to get and set a 
/// variety of properties. Some of these properties are used in syncronization
/// and timing calculations.</remarks>
/// TODO checkout which of these are really needed
public class SubtitleProperties {
	private SubtitleHeaders headers = new SubtitleHeaders();
	
	private float originalFrameRate = 25;
	private float currentFrameRate = 25;
	private TimingMode timingMode = TimingMode.Times;

	private string filePath = String.Empty;
	private Encoding fileEncoding = Encoding.Default;
	private SubtitleType subtitleType = SubtitleType.Unknown;
	
	/// <summary>Initializes a new instance of the <see cref="SubtitleProperties" />
	/// class, with defaults for all properties.</summary>
	public SubtitleProperties () {
	}
	
	
	/// <summary>The headers used in some subtitle formats.</summary>
	public SubtitleHeaders Headers {
		get { return headers; }
	}
		
	/// <summary>The frame rate originally applied to the subtitles.</summary>
	/// <remarks>When converting between frame rates, this is the frame rate of the subtitles
	/// when they are opened. This is sometimes refered to as the input frame rate.</remarks>
	public float OriginalFrameRate {
		get { return originalFrameRate; }
	}
	
	/// <summary>The frame rate currently being used in the subtitles.</summary>
	/// <remarks>When converting between frame rates, this is the target frame rate of the
	/// subtitles. This is sometimes refered to as the output frame rate.</remarks>
	public float CurrentFrameRate {
		get { return currentFrameRate; }
	}
		
	/// <summary>The subtitles' file name.</summary>
	/// <remarks>Only set the file name with this property when the file's absolute path is
	/// unknown (i.e., the file has a name but no path). Setting the file name will clear any
	/// existing absolute path. Use <see cref="FilePath" /> if the absolute path is known.</remarks>
	public string FileName {
		get { return Path.GetFileName(filePath); }
		set { filePath = value; }
	}
	
	/// <summary>The absolute path of the subtitles' file.</summary>
	/// <remarks>The result excludes the file name and the last <see cref="Path.DirectorySeparatorChar" />.
	/// See <see cref="Path.GetDirectoryName" /> for additional information.</remarks>
	public string FileDirectory {
		get { return Path.GetDirectoryName(filePath); }
	}
	
	/// <summary>The subtitle's absolute file path.</summary>
	/// <remarks>The result may consist on only the file name, if the absolute path isn't
	/// known. A call to <see cref="IsFilePathRooted" /> can be used to check if the path is
	/// absolute.</remarks>
	public string FilePath {
		get { return filePath; }
		set { filePath = Path.GetFullPath(value); }
	}
	
	/// <summary>Whether the subtitles' file path is absolute (i.e., is rooted).</summary>
	/// <remarks>See <see cref="Path.IsPathRooted" /> for additional information.</remarks>
	public bool IsFilePathRooted {
		get { return Path.IsPathRooted(filePath); }
	}
	
	/// <summary>The character encoding used in the subtitles' file.</summary>
	public Encoding FileEncoding {
		get { return fileEncoding; }
		set { fileEncoding = value; }
	}
	
	/// <summary>The timing mode currently in use with the subtitles.</summary>
	public TimingMode TimingMode {
		get { return timingMode; }
		set { timingMode = value; }
	}
	
	/// <summary>The type of the subtitles' file format.</summary>
	/// <remarks>This is mostly used to remember the type of the subtitles' file before
	/// opened, so it can be saved using the same type.</remarks>
	public SubtitleType SubtitleType {
		get { return subtitleType; }
		set { subtitleType = value; }
	}
	
	public override string ToString(){
		return "\t* SUBTITLE PROPERTIES *\n" +
			"Original Frame Rate: " + originalFrameRate + ", Current Frame Rate: " + currentFrameRate +
			", Timing Mode: " + timingMode + ", Path: " + filePath + ", Encoding: " + fileEncoding +
			", SubtitleType: " + subtitleType + "\n" + headers.ToString();
	}
	
	/* Internal members */
	
	internal void SetCurrentFrameRate (float frameRate) {
		currentFrameRate = frameRate;
	}
	
	internal void SetOriginalFrameRate (float frameRate) {
		originalFrameRate = frameRate;
	}
	
}

}
